# -*- coding: utf_8 -*-
from __future__ import with_statement
print "b92"
os.chdir("/home/brettnak/.sage/sage_notebook/worksheets/admin/5/cells/1")
sage.server.notebook.interact.SAGE_CELL_ID=1
class Sensor_Network:
    """
__init__(self, n, k, p, area, range)
    n = number of nodes
    k = size of key rings
    p = size of key pool
    area = (xmin, xmax, ymin, ymax)

add_nodes(nodes)
    nodes - a list or tuple in the form ((key_ring, location, name), Ellipsis)
        key_ring - *list* of keys defined in the current key pool
                       **note, does not have to be in key pool, but should be (or what's the point?)
            location - must be a tuple (x, y), defined in the current area, or None
            name     - must be a any object with a __repr__ or None

add_node(node)
    parameters -
        node - iterable object in form (key_ring, location, name)
            key_ring - *list* of keys defined in the current key pool
                       **note, does not have to be in key pool, but should be (or what's the point?)
            location - must be a tuple (x, y), defined in the current area, or None
            name     - must be a any object with a __repr__ or None
        """
    def __init__(self, n, k, p, area, range = Integer(1)):
        pool = (ellipsis_range(Integer(0),Ellipsis,p-Integer(1)))
        self.key_pool = pool
        self.area = area
        xvals = r.runif(n, area[Integer(0)], area[Integer(1)])
        yvals = r.runif(n, area[Integer(2)], area[Integer(3)])
        self.yvals = yvals
        self.xvals = xvals
        self.range = range
        self.nodes = []
        self.links = []
        self.k = k
        self.redraw = true
        self.compromised_keys = set([])
        for i in (ellipsis_range(Integer(0),Ellipsis,n-Integer(1))):
            key_ring = r.sample(pool, k)
            if k == Integer(1): key_ring = [key_ring]
            node = Sensor(i, key_ring, (xvals[i], yvals[i]))
            for j in self.nodes:
                if node.is_in_range(j, self.range):
                    common_keys = j.get_common_keys(node)
                    if len(common_keys) > Integer(0):
                        node.add_neighbor(j)
                        j.add_neighbor(node)
                        self.links.append(Link(j, node, j.get_distance(node), common_keys));
            self.nodes.append(node)

    def get_sample_keyring(self, k = None):
        if k == None: k = self.k
        return r.sample(self.key_pool, k)

    def add_nodes(self, n):
        for i in (ellipsis_range(Integer(1),Ellipsis,n)):
            self.add_node((self.get_sample_keyring(), None, None))

    def add_node(self, node):
        self.redraw = true
        location = node[Integer(1)]
        key_ring = node[Integer(0)]
        name = node[Integer(2)]
        if location == None:
            x = r.runif(Integer(1), Integer(0), self.area[Integer(1)])
            y = r.runif(Integer(1), Integer(0), self.area[Integer(3)])
            location = (x, y)
        else:
            if not(location[Integer(0)] > Integer(0) and location[Integer(0)] < area[Integer(1)]):
                message = "location is invalid (out of bounds in x)"
                raise ValueError, message
            elif not(location[Integer(1)] > Integer(0) and location[Integer(1)] < area[Integer(3)]):
                message = "location is invalid (out of bounds in y)"
                raise ValueError, message
        toadd = Sensor(len(self.nodes), key_ring, location, name, None)
        for i in self.nodes:
            if toadd.is_in_range(i, self.range):
                common_keys = toadd.get_common_keys(i)
                if len(common_keys) > Integer(0):
                    newlink = Link(i, toadd, i.get_distance(toadd), common_keys)
                    newlink.remove_keys(self.compromised_keys)
                    self.links.append(newlink)
        self.nodes.append(toadd)

    def capture_node(self, nodeid):
        # get keys from the node
        # look through all links for links consisting of only keys which have been comprimised.
        to_capture = None
        for node in self.nodes:
            if node.id == nodeid:
                self.capture_node_priv(node)
                break

    def capture_node_priv(self, to_capture):
        self.redraw = true
        key_ring = to_capture.compromise()
        key_ring_set = set(key_ring)
        self.compromised_keys = self.compromised_keys.union(key_ring_set)
        for link in self.links:
            link.remove_keys(key_ring)

    def capture_optimal(self):
        node_degrees = {}
        maxnode = self.nodes[Integer(0)]
        maxdegree = Integer(0)
        for link in self.links:
            if not link.compromised:
                try:
                    node_degrees[link.nodea] += Integer(1)
                    if maxdegree < node_degrees[link.nodea]:
                        maxdegree = node_degrees[link.nodea]
                        maxnode = link.nodea
                except KeyError:
                    node_degrees[link.nodea] = Integer(0)
                try:
                    node_degrees[link.nodeb] += Integer(1)
                    if maxdegree < node_degrees[link.nodeb]:
                        maxdegree = node_degrees[link.nodeb]
                        maxnode = link.nodeb
                except KeyError:
                    node_degrees[link.nodeb] = Integer(0)
        self.capture_node_priv(maxnode)


    def show(self, f_size=Integer(10), draw = true):
        if not self.redraw:
            if draw:
                G = self.graph_cache[Integer(0)]
                v_colors = self.graph_cache[Integer(1)]
                e_colors = self.graph_cache[Integer(2)]
                positions = self.graph_cache[Integer(3)]
                G.show(vertex_colors = v_colors, edge_colors = e_colors, pos = positions, figsize=f_size)
            return self.graph_cache
        G = graphs.EmptyGraph()
        v_colors = {'red':[], 'green':[]}
        e_colors = {'red':[], 'green':[]}
        positions = {}
        for i in self.nodes:
            G.add_vertex(i.id)
            positions[i.id] = list(i.location)
            if i.compromised: v_colors['red'].append(i.id)
            else: v_colors['green'].append(i.id)
        for e in self.links:
            if e.compromised: e_colors['red'].append((e.nodea.id, e.nodeb.id))
            else: e_colors['green'].append((e.nodea.id, e.nodeb.id))
        if draw:
            G.show(vertex_colors = v_colors, edge_colors = e_colors, pos = positions, figsize=f_size)
        self.redraw = false
        self.graph_cache = (G, v_colors, e_colors, positions)
        return (G, v_colors, e_colors, positions)

print "e92"
