# -*- coding: utf_8 -*-
from __future__ import with_statement
print "b101"
os.chdir("/home/brettnak/.sage/sage_notebook/worksheets/admin/5/cells/0")
sage.server.notebook.interact.SAGE_CELL_ID=0
from rpy import *

def create_sensor_network_neighborhood(n, k, r, h, c, m, v):
    """
Returns a Sensor_Network object with an approximate
connectivity c, given a number of nodes, n, with key
rings of size k, and a neighborhood h, with a sensor
range of r.

parameters (n, k, r, h, c, m)
    n =     number of nodes
    k =     size of keyrings
    r =     range of each node
    h =     neighborhood size (nodes)
    c =     desired connectivity
    m =     ratio of the rectangle ( m * height =  width )
    v =     boolean value of whether or not to print out the network's specifications
    """
    a = n * pi * r**Integer(2) / h               # The area of the graph based on neighborhood size
    a = float(a)
    p = -log(Integer(1) - c**(Integer(1) / n)) / h        # The probability of connection to a neighboring node
                                       # This is for an infinite graph

    if c > float((Integer(1) - e ** (-h)) ** n):
        message = "maximum c value is (1 - e ^ (-h))^n, your value was " + repr(float((Integer(1) - e ** (-h)) ** n))
        raise ValueError, message
    p = RealNumber(p)
    P = k / (Integer(1) - (Integer(1) - p) ** (Integer(1) / k))  # The size of the key pool
    P = P.floor()
    n = Integer(n)
    k = Integer(k)
    height = sqrt(a / m)
    width = m * height
    area = (Integer(0), width, Integer(0), height)
    if v:
        print "Number of Nodes           = " + repr(n)
        print "Size of Neighborhood      = " + repr(h)
        print "Desired Connectivity      = " + repr(c)
        print "Key Ring Size             = " + repr(k)
        print "Small p                   = " + repr(p)
        print "    probability of connection to a neighboring node"
        print "Key Pool Size             = " + repr(P)
        print "Radio Range               = " + repr(r)
        print "Area                      = " + repr(a)
        print "Dimensions of Area:"
        print "    x-min                 = " + repr(area[Integer(0)])
        print "    x-max                 = " + repr(area[Integer(1)])
        print "    y-min                 = " + repr(area[Integer(2)])
        print "    y-max                 = " + repr(area[Integer(3)])
    return Sensor_Network(n, k, P, area, r)

def create_sensor_network_area(n, k, r, a, c):
    """
Returns a new sensor network object with approximate
connectivity c, based on n nodes, k keys per node,
sensor range r, in an area defined by a.

    parameters (n, k, r, a, c)
        n =     number of nodes
        k =     size of keyrings
        r =     range of each node
        a =     area tuple (xmin, xmax, ymin, ymax)
        c =     desired connectivity
        r =     ratio of the rectangle ( r * height =  width )
    """
    area = (( a[Integer(1)] - a[Integer(0)]) ** Integer(2))
    area *= ((a[Integer(3)] - a[Integer(1)]) ** Integer(2))
    area = float(a)
    h = n * pi * r**Integer(2) / area          # The neighborhood of a graph based on it's area
    h = h.floor()                    # Round down to guarantee conductivity c
    p = -log(Integer(1) - c**(Integer(1) / n)) / h      # The probability of connection to a neighboring node
                                     # This is for an infinite graph
    p = float(p)
    P = k / (Integer(1) - (Integer(1) - p) ** (Integer(1) / k))  # The size of the key pool
    P = P.floor()
    n = Integer(n)
    k = Integer(k)
    return Sensor_Network(n, k, P, a, r)

def show_secure(a, f_size = Integer(10)):
    """
a = The tuple returned by Sensor_Network.show()
f_size = figsize parameter for Graph.show(), default 10
    """
    a[Integer(0)].show(edge_colors = {'green':a[Integer(2)]['green']}, vertex_colors = a[Integer(1)], pos = a[Integer(3)], figsize = f_size)

def show_compromised(a, f_size = Integer(10)):
    """
a = The tuple returned by Sensor_Network.show()
f_size = figsize parameter for Graph.show(), default 10
    """
    a[Integer(0)].show(edge_colors = {'red':a[Integer(2)]['red']}, vertex_colors = a[Integer(1)], pos = a[Integer(3)], figsize = f_size)

print "e101"
